Class: MTProto::Updates::Parser::Batch

Inherits:
Struct
  • Object
show all
Defined in:
lib/mtproto/updates/parser.rb

Overview

Batch is the live-container result; Difference is the recovery-reply result and additionally carries the new authoritative position (pts/qts/date/seq), or, for a channel, the new channel pts + final (whether the backlog is drained).

Instance Attribute Summary collapse

Instance Attribute Details

#channel_too_longObject

Returns the value of attribute channel_too_long

Returns:

  • (Object)

    the current value of channel_too_long



38
39
40
# File 'lib/mtproto/updates/parser.rb', line 38

def channel_too_long
  @channel_too_long
end

#chatsObject

Returns the value of attribute chats

Returns:

  • (Object)

    the current value of chats



38
39
40
# File 'lib/mtproto/updates/parser.rb', line 38

def chats
  @chats
end

#dateObject

Returns the value of attribute date

Returns:

  • (Object)

    the current value of date



38
39
40
# File 'lib/mtproto/updates/parser.rb', line 38

def date
  @date
end

#eventsObject

Returns the value of attribute events

Returns:

  • (Object)

    the current value of events



38
39
40
# File 'lib/mtproto/updates/parser.rb', line 38

def events
  @events
end

#too_longObject

Returns the value of attribute too_long

Returns:

  • (Object)

    the current value of too_long



38
39
40
# File 'lib/mtproto/updates/parser.rb', line 38

def too_long
  @too_long
end

#usersObject

Returns the value of attribute users

Returns:

  • (Object)

    the current value of users



38
39
40
# File 'lib/mtproto/updates/parser.rb', line 38

def users
  @users
end